@@ -5,7 +5,7 @@ oer - IRC bot
 See ftp://nic.funet.fi/pub/unix/irc/docs/FAQ.gz section 11 for the
 definition of the word bot.
 
-Copyright (C) 2000-2001 EQU <equ@equnet.org>
+Copyright (C) 2000-2002 EQU <equ@equnet.org>
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -124,14 +124,17 @@ extern int deltaflags(struct channel *, char *, char);
 extern int isvalidchannel(char *);
 extern int hasuserflags(struct channel *, char *, char *, char *);
 extern int noexpiredlogons(struct channel *);
+extern int delchannelban(struct channel *, char *);
 extern unsigned int getrandom(unsigned int);
 extern long getfilesize(char *);
 extern struct authed *addnewauthed(struct channel *, time_t, struct botuser *, char *);
 extern struct botuser *addbotuser(char *, char *);
 extern struct botuser *addnewadmin(char *, char *);
 extern struct botuser *addnewuser(struct channel *, char *, char *);
+extern struct botuser *usercopy(struct channel *, char *, struct channel *, char *);
 extern struct channel *addnewchannel(char *);
 extern struct channel *getchptr(char *);
+extern struct channelban *addnewchannelban(struct channel *, char *, char *, time_t);
 extern struct chanuser *userjoined(struct channel *, char *, char *, int, int, int, int);
 extern struct chanuser *getcuptr(struct channel *, char *);
 extern struct state *emptystate(void);
@@ -178,6 +181,7 @@ extern void initjoins(struct channel *);
 extern void initnickbks(struct channel *);
 extern void initwordbks(struct channel *);
 extern void initpermbans(struct channel *);
+extern void initchannelbans(struct channel *);
 extern void initusers(struct channel *);
 extern void delalltopics(struct channel *);
 extern void swaptopic(struct channel *, int, int);
@@ -230,3 +234,5 @@ extern void cleanautheds(struct channel *, int);
 extern void listchanusers(void);
 extern void timed_del(struct timed *);
 extern void mmode_del(struct channel *, struct mmode *);
+extern void validateflags(char *, char *, int);
+extern void resetfloodcounters(struct channel *, char *);
